@@ -1,8 +1,9 @@
 from typing import List
 
-from mstrio.utils.helper import response_handler
+from mstrio.utils.error_handlers import ErrorHandler
 
 
+@ErrorHandler(err_msg='Error getting bookmarks from shortcut {shortcut_id}')
 def get_bookmarks_from_shortcut(connection, shortcut_id, error_msg=None):
     """Get a Bookmark list from a Shortcut Object.
 
@@ -14,15 +15,11 @@ def get_bookmarks_from_shortcut(connection, shortcut_id, error_msg=None):
     Returns:
         Complete HTTP response object.
     """
-    url = connection.base_url + f"/api/shortcuts/{shortcut_id}/bookmarks"
-    response = connection.session.get(url=url)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error getting bookmarks from shortcut {shortcut_id}"
-        response_handler(response, error_msg)
-    return response
+    url = f"{connection.base_url}/api/shortcuts/{shortcut_id}/bookmarks"
+    return connection.session.get(url=url)
 
 
+@ErrorHandler(err_msg='Error updating information for shortcut {shortcut_id}')
 def update_information_for_shortcut(connection, shortcut_id, body, error_msg=None):
     """Update info for a shortcut.
 
@@ -35,14 +32,10 @@ def update_information_for_shortcut(connection, shortcut_id, body, error_msg=Non
         Complete HTTP response object.
     """
     url = connection.base_url + f"​/api/shortcuts/{shortcut_id}"
-    response = connection.session.patch(url=url, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error updating information for shortcut {shortcut_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.patch(url=url, json=body)
 
 
+@ErrorHandler(err_msg='Error adding a new bookmark.')
 def refresh_document_instance(connection, error_msg=None):
     """Add a new bookmark into current shortcut object.
 
@@ -54,14 +47,10 @@ def refresh_document_instance(connection, error_msg=None):
         Complete HTTP response object.
     """
     url = connection.base_url + "​/api​/bookmarks"
-    response = connection.session.put(url=url)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = "Error adding a new bookmark"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.put(url=url)
 
 
+@ErrorHandler(err_msg='Error deleting bookmarks.')
 def delete_bookmarks(connection, shortcut_id: str, bookmark_ids: List, error_msg=None):
     """Bulk deletion of bookmarks.
 
@@ -75,15 +64,11 @@ def delete_bookmarks(connection, shortcut_id: str, bookmark_ids: List, error_msg
         Complete HTTP response object.
     """
     body = {"shortcutId": shortcut_id, "bookmarkIds": bookmark_ids}
-    url = connection.base_url + "/api/bookmarks"
-    response = connection.session.delete(url=url, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = "Error deleting bookmarks"
-        response_handler(response, error_msg)
-    return response
+    url = f'{connection.base_url}/api/bookmarks'
+    return connection.session.delete(url=url, json=body)
 
 
+@ErrorHandler(err_msg='Error deleting bookmark with ID {bookmark_id}')
 def delete_single_bookmark(connection, shortcut_id: str, bookmark_id: str, error_msg=None):
     """Delete a bookmark.
 
@@ -97,15 +82,11 @@ def delete_single_bookmark(connection, shortcut_id: str, bookmark_id: str, error
         Complete HTTP response object.
     """
     body = {"shortcutId": shortcut_id}
-    url = connection.base_url + f"/api/bookmarks/{bookmark_id}"
-    response = connection.session.delete(url=url, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error deleting bookmark {bookmark_id}"
-        response_handler(response, error_msg)
-    return response
+    url = f'{connection.base_url}/api/bookmarks/{bookmark_id}'
+    return connection.session.delete(url=url, json=body)
 
 
+@ErrorHandler(err_msg='Error updating bookmark {bookmark_id}')
 def update_bookmark(connection, bookmark_id, body, error_msg=None):
     """Update a bookmark.
 
@@ -118,15 +99,11 @@ def update_bookmark(connection, bookmark_id, body, error_msg=None):
     Returns:
         Complete HTTP response object.
     """
-    url = connection.base_url + f"/api/bookmarks/{bookmark_id}"
-    response = connection.session.put(url=url, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error updating bookmark {bookmark_id}"
-        response_handler(response, error_msg)
-    return response
+    url = f"{connection.base_url}/api/bookmarks/{bookmark_id}"
+    return connection.session.put(url=url, json=body)
 
 
+@ErrorHandler(err_msg='Error adding bookmark {bookmark_name}')
 def add_bookmark(connection, bookmark_name, instance_id, shortcut_id,
                  error_msg=None):  # shortcut needs to be created to add a bookmark!
     """Update a bookmark.
@@ -139,15 +116,11 @@ def add_bookmark(connection, bookmark_name, instance_id, shortcut_id,
         Complete HTTP response object.
     """
     body = {"name": bookmark_name, "instanceId": instance_id}
-    url = connection.base_url + "/api/bookmarks"
-    response = connection.session.post(url=url, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = "Error adding bookmark"
-        response_handler(response, error_msg)
-    return response
+    url = f'{connection.base_url}/api/bookmarks'
+    return connection.session.post(url=url, json=body)
 
 
+@ErrorHandler(err_msg='Error getting shortcuts for document {document_id}')
 def get_document_shortcut(connection, document_id, instance_id, error_msg=None):
     """Retrieve a published shortcut from the document definition.
 
@@ -162,9 +135,4 @@ def get_document_shortcut(connection, document_id, instance_id, error_msg=None):
         Complete HTTP response object.
     """
     endpoint_url = f'/api/documents/{document_id}/instances/{instance_id}/shortcut'
-    response = connection.session.get(connection.base_url + endpoint_url)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error getting shortcuts for document {document_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(connection.base_url + endpoint_url)
