@@ -1,13 +1,16 @@
 from typing import Optional
+import json
 
 from mstrio.connection import Connection
-from mstrio.server.application import Application
+from mstrio.server.project import Project
 from mstrio.utils.datasources import (alter_conn_list_resp, alter_conn_resp,
                                       alter_instance_list_resp, alter_instance_resp,
                                       alter_patch_req_body)
-from mstrio.utils.helper import exception_handler, response_handler
+from mstrio.utils.error_handlers import ErrorHandler
+from mstrio.utils.helper import exception_handler, response_handler, deprecation_warning
 
 
+@ErrorHandler(err_msg='Error getting available DBMSs.')
 def get_available_dbms(connection, error_msg=None):
     """Get information for all available database management systems (DBMSs).
 
@@ -19,14 +22,10 @@ def get_available_dbms(connection, error_msg=None):
         Complete HTTP response object. HTTP STATUS 200/400
     """
     url = f"{connection.base_url}/api/dbobjects/dbmss"
-    response = connection.session.get(url=url)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = "Error getting available DBMSs"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url)
 
 
+@ErrorHandler(err_msg='Error getting available database drivers.')
 def get_available_db_drivers(connection, error_msg=None):
     """Get information for all available database drivers.
 
@@ -38,12 +37,7 @@ def get_available_db_drivers(connection, error_msg=None):
         Complete HTTP response object. HTTP STATUS 200/400
     """
     url = f"{connection.base_url}/api/dbobjects/drivers"
-    response = connection.session.get(url=url)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = "Error getting available database drivers"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url)
 
 
 def get_datasource_instance(connection, id, error_msg=None):
@@ -67,6 +61,7 @@ def get_datasource_instance(connection, id, error_msg=None):
     return response
 
 
+@ErrorHandler(err_msg='Error deleting Datasource Instance with ID {id}')
 def delete_datasource_instance(connection, id, error_msg=None):
     """Delete a specific database source based on id.
 
@@ -79,12 +74,7 @@ def delete_datasource_instance(connection, id, error_msg=None):
         Complete HTTP response object. HTTP STATUS 204/400
     """
     url = f"{connection.base_url}/api/datasources/{id}"
-    response = connection.session.delete(url=url)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error deleting Datasource Instance with ID: {id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.delete(url=url)
 
 
 def update_datasource_instance(connection, id, body, error_msg=None):
@@ -138,15 +128,16 @@ def create_datasource_instance(connection, body, error_msg=None):
     return response
 
 
-def get_datasource_instances(connection, ids=None, database_type=None, application=None,
-                             error_msg=None):
+def get_datasource_instances(connection, ids=None, database_type=None, project=None,
+                             error_msg=None, application=None):
     """Get information for all database sources.
 
     Args:
         connection: MicroStrategy REST API connection object
         ids: list of datasources ids
         database_type: list of types (names) of databases
-        application: id (str) of a project or instance of an Application class
+        application: deprecated. Use project instead.
+        project: id (str) of a project or instance of an Project class
             to search for the datasource instances in. When provided, both
             `ids` and `database_types` are ignored. By default `None`.
         error_msg (string, optional): Custom Error Message for Error Handling
@@ -154,11 +145,18 @@ def get_datasource_instances(connection, ids=None, database_type=None, applicati
     Returns:
         Complete HTTP response object. HTTP STATUS 200/400
     """
-    application = application.id if isinstance(application, Application) else application
-    application_provided = application is not None
+    if application:
+        deprecation_warning(
+            '`application`',
+            '`project`',
+            '11.3.4.101',  # NOSONAR
+            False)
+        project = project or application
+    project = project.id if isinstance(project, Project) else project
+    project_provided = project is not None
 
-    if application_provided:
-        url = f"{connection.base_url}/api/projects/{application}/datasources"
+    if project_provided:
+        url = f"{connection.base_url}/api/projects/{project}/datasources"
         response = connection.session.get(url=url)
     else:
         database_type = None if database_type is None else database_type.join(",")
@@ -170,24 +168,24 @@ def get_datasource_instances(connection, ids=None, database_type=None, applicati
         })
     if not response.ok:
         res = response.json()
-        if application_provided and res.get("message") == "HTTP 404 Not Found":
-            # aka application based endpoint not supported
+        if project_provided and res.get("message") == "HTTP 404 Not Found":
+            # aka project based endpoint not supported
             # try without filtering
-            warning_msg = ("get_datasource_instances() warning: filtering by Application "
+            warning_msg = ("get_datasource_instances() warning: filtering by Project "
                            "is not yet supported on this version of the I-Server. "
                            "Returning all values.")
             exception_handler(warning_msg, Warning, 0)
             return get_datasource_instances(connection=connection, ids=ids,
                                             database_type=database_type, error_msg=error_msg)
         if error_msg is None:
-            if application_provided \
+            if project_provided \
                     and res.get('code') == "ERR006" \
                     and "not a valid value for Project ID" in res.get('message'):
-                error_msg = f"{application} is not a valid Application class instance or ID"
+                error_msg = f"{project} is not a valid Project class instance or ID"
                 raise ValueError(error_msg)
             error_msg = "Error getting Datasource Instances"
-            if application_provided:
-                error_msg += f" within `{application}` Application"
+            if project_provided:
+                error_msg += f" within `{project}` Project"
         response_handler(response, error_msg)
     response = alter_instance_list_resp(response)
     return response
@@ -258,7 +256,8 @@ def update_datasource_connection(connection, id, body, error_msg=None):
     return response
 
 
-def delete_datasource_connection(connection, id, error_msg=None):
+@ErrorHandler(err_msg='Error deleting Datasource Connection with ID {id}')
+def delete_datasource_connection(connection, id):
     """Delete a datasource connection based on id.
 
     Args:
@@ -270,12 +269,7 @@ def delete_datasource_connection(connection, id, error_msg=None):
         Complete HTTP response object. HTTP STATUS 204/400
     """
     url = f"{connection.base_url}/api/datasources/connections/{id}"
-    response = connection.session.delete(url=url)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error deleting Datasource Connection with ID: {id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.delete(url=url)
 
 
 def create_datasource_connection(connection, body, error_msg=None):
@@ -300,6 +294,7 @@ def create_datasource_connection(connection, body, error_msg=None):
     return response
 
 
+@ErrorHandler(err_msg='Error testing Datasource connection.')
 def test_datasource_connection(connection, body, error_msg=None):
     """Test a datasource connection. Either provide a connection id, or the
     connection parameters within connection object.
@@ -313,24 +308,59 @@ def test_datasource_connection(connection, body, error_msg=None):
         Complete HTTP response object. HTTP STATUS 204/400
     """
     url = f"{connection.base_url}/api/datasources/connections/test"
-    response = connection.session.post(url=url, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = "Error testing Datasource connection"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.post(url=url, json=body)
 
 
+@ErrorHandler(err_msg='Error fetching Datasource mappings.')
 def get_datasource_mappings(connection: Connection, default_connection_map: Optional[bool] = False,
-                            application_id: Optional[str] = None, error_msg: Optional[str] = None):
+                            project_id: Optional[str] = None, error_msg: Optional[str] = None,
+                            application_id: Optional[str] = None):
     """Get information for all datasource connection mappings.
 
     Args:
         connection: MicroStrategy REST API connection object
         default_connection_map (bool, optional): If True will get the default
-            connection map for an application. Requires `application_id`
+            connection map for an project. Requires `project_id`
             parameter. Default False.
-        application_id: The application_id, required only for default connection
+        project_id: The project_id, required only for default connection
+            map.
+        application_id: deprecated. Use project_id instead.
+        error_msg (string, optional): Custom Error Message for Error Handling
+
+    Returns:
+        Complete HTTP response object. Expected status is 200.
+    """
+    if application_id:
+        deprecation_warning(
+            '`application_id`',
+            '`project_id`',
+            '11.3.4.101',  # NOSONAR
+            False)
+        project_id = project_id or application_id
+    url = f"{connection.base_url}/api/datasources/mappings"
+    return connection.session.get(
+        url=url, params={
+            "defaultConnectionMap": default_connection_map,
+            "projectId": project_id
+        })
+
+
+# This is a 'fake' get single resource endpoint. Only listing all mappings
+# is available on REST API.
+# TODO Change to normal get, when it will be available on REST.
+@ErrorHandler(err_msg='Error fetching Datasource mapping with ID {id}.')
+def get_datasource_mapping(connection: Connection, id=str,
+                           default_connection_map: Optional[bool] = False,
+                           project_id: Optional[str] = None, error_msg: Optional[str] = None):
+    """Get information about specific datasource_mapping.
+
+    Args:
+        connection: MicroStrategy REST API connection object
+        id: ID of the mapping
+        default_connection_map (bool, optional): If True will get the default
+            connection map for an application. Requires `project_id`
+            parameter. Default False.
+        project_id: The project_id, required only for default connection
             map.
         error_msg (string, optional): Custom Error Message for Error Handling
 
@@ -341,15 +371,22 @@ def get_datasource_mappings(connection: Connection, default_connection_map: Opti
     response = connection.session.get(
         url=url, params={
             "defaultConnectionMap": default_connection_map,
-            "projectId": application_id
+            "projectId": project_id
         })
-    if not response.ok:
-        if error_msg is None:
-            error_msg = "Error fetching Datasource mappings"
-        response_handler(response, error_msg)
+
+    # Faking get single resource endpoint. Only 'list all' available on REST
+    if response.ok:
+        response_json = response.json()
+        if 'mappings' in response_json.keys():
+            mappings = [mapping for mapping in response_json['mappings'] if mapping["id"] == id]
+            if len(mappings) > 0:
+                response_json = mappings[0]
+                response_json['ds_connection'] = response_json.pop('connection')
+        response.encoding, response._content = 'utf-8', json.dumps(response_json).encode('utf-8')
     return response
 
 
+@ErrorHandler(err_msg='Error creating Datasource mapping.')
 def create_datasource_mapping(connection: Connection, body, error_msg: Optional[str] = None):
     """Create a new datasource mapping.
 
@@ -362,14 +399,10 @@ def create_datasource_mapping(connection: Connection, body, error_msg: Optional[
         Complete HTTP response object. Expected status is 201.
     """
     url = f"{connection.base_url}/api/datasources/mappings"
-    response = connection.session.post(url=url, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = "Error creating Datasource mapping"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.post(url=url, json=body)
 
 
+@ErrorHandler(err_msg='Error deleting Datasource mapping with ID {id}')
 def delete_datasource_mapping(connection: Connection, id: str, error_msg: Optional[str] = None):
     """Delete a datasource mapping based on id.
 
@@ -382,14 +415,10 @@ def delete_datasource_mapping(connection: Connection, id: str, error_msg: Option
         Complete HTTP response object. Expected status is 204.
     """
     url = f"{connection.base_url}/api/datasources/mappings/{id}"
-    response = connection.session.delete(url=url)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = "Error deleting Datasource mapping"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.delete(url=url)
 
 
+@ErrorHandler(err_msg="Error getting Datasource logins.")
 def get_datasource_logins(connection: Connection, error_msg: Optional[str] = None):
     """Get information for all datasource logins.
 
@@ -401,14 +430,10 @@ def get_datasource_logins(connection: Connection, error_msg: Optional[str] = Non
         Complete HTTP response object. Expected status is 200.
     """
     url = f"{connection.base_url}/api/datasources/logins"
-    response = connection.session.get(url=url)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = "Error getting Datasource logins"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url)
 
 
+@ErrorHandler(err_msg='Error creating Datasource login.')
 def create_datasource_login(connection: Connection, body, error_msg: Optional[str] = None):
     """Create a new datasource login.
 
@@ -421,14 +446,10 @@ def create_datasource_login(connection: Connection, body, error_msg: Optional[st
         Complete HTTP response object. Expected status is 201.
     """
     url = f"{connection.base_url}/api/datasources/logins"
-    response = connection.session.post(url=url, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = "Error creating Datasource login"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.post(url=url, json=body)
 
 
+@ErrorHandler(err_msg='Error getting Datasource login with ID {id}')
 def get_datasource_login(connection: Connection, id: str, error_msg: Optional[str] = None):
     """Get datasource login for a specific id.
 
@@ -441,14 +462,10 @@ def get_datasource_login(connection: Connection, id: str, error_msg: Optional[st
         Complete HTTP response object. Expected status is 200.
     """
     url = f"{connection.base_url}/api/datasources/logins/{id}"
-    response = connection.session.get(url=url)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error getting Datasource login with ID {id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url)
 
 
+@ErrorHandler(err_msg='Error deleting Datasource login with ID {id}')
 def delete_datasource_login(connection: Connection, id: str, error_msg: Optional[str] = None):
     """Delete a datasource login.
 
@@ -461,14 +478,10 @@ def delete_datasource_login(connection: Connection, id: str, error_msg: Optional
         Complete HTTP response object. Expected status is 204.
     """
     url = f"{connection.base_url}/api/datasources/logins/{id}"
-    response = connection.session.delete(url=url)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error deleting Datasource login with ID {id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.delete(url=url)
 
 
+@ErrorHandler(err_msg='Error updating Datasource login with ID {id}')
 def update_datasource_login(connection: Connection, id: str, body,
                             error_msg: Optional[str] = None):
     """Update a datasource login.
@@ -483,9 +496,4 @@ def update_datasource_login(connection: Connection, id: str, body,
         Complete HTTP response object. Expected status is 200.
     """
     url = f"{connection.base_url}/api/datasources/logins/{id}"
-    response = connection.session.patch(url=url, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = "Error creating Datasource login"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.patch(url=url, json=body)
