@@ -1,6 +1,12 @@
-from mstrio.utils.helper import response_handler
+from typing import TYPE_CHECKING
+
+from mstrio.utils.error_handlers import ErrorHandler
+
+if TYPE_CHECKING:
+    from requests_futures.sessions import FuturesSession
 
 
+@ErrorHandler(err_msg='Error getting available dossiers.')
 def get_dossiers(connection, search_term, certified_status, search_pattern='EXACTLY', offset=0,
                  limit=-1, fields=None, error_msg=None):
     """Get the list of available dossiers.
@@ -12,8 +18,8 @@ def get_dossiers(connection, search_term, certified_status, search_pattern='EXAC
     Returns:
         Complete HTTP response object.
     """
-    url = connection.base_url + "/api/dossiers"
-    response = connection.session.get(
+    url = f'{connection.base_url}/api/dossiers'
+    return connection.session.get(
         url=url,
         params={
             'searchTerm': search_term,
@@ -23,14 +29,9 @@ def get_dossiers(connection, search_term, certified_status, search_pattern='EXAC
             'fields': fields
         },
     )
-    if not response.ok:
-        if error_msg is None:
-            error_msg = "Error getting available dossiers"
-        response_handler(response, error_msg)
-    return response
 
 
-def get_dossiers_async(future_session, connection, search_term, certified_status,
+def get_dossiers_async(future_session: "FuturesSession", connection, search_term, certified_status,
                        search_pattern='EXACTLY', offset=0, limit=-1, fields=None, error_msg=None):
     """Get the list of available dossiers asynchronously.
 
@@ -41,8 +42,8 @@ def get_dossiers_async(future_session, connection, search_term, certified_status
     Returns:
         Complete HTTP response object.
     """
-    connection._validate_application_selected()
-    url = connection.base_url + "/api/dossiers"
+    connection._validate_project_selected()
+    url = f'{connection.base_url}/api/dossiers'
     params = {
         'searchTerm': search_term,
         'searchPattern': search_pattern,
@@ -50,10 +51,10 @@ def get_dossiers_async(future_session, connection, search_term, certified_status
         'limit': limit,
         'fields': fields
     }
-    future = future_session.get(url=url, params=params)
-    return future
+    return future_session.get(url=url, params=params)
 
 
+@ErrorHandler(err_msg='Error getting available documents.')
 def get_documents(connection, search_term, certified_status, search_pattern='EXACTLY', offset=0,
                   limit=-1, fields=None, error_msg=None):
     """Get the list of available documents.
@@ -65,9 +66,9 @@ def get_documents(connection, search_term, certified_status, search_pattern='EXA
     Returns:
         Complete HTTP response object.
     """
-    url = connection.base_url + '/api/documents/'
-    response = connection.session.get(
-        url,
+    url = f'{connection.base_url}/api/documents/'
+    return connection.session.get(
+        url=url,
         params={
             'searchTerm': search_term,
             'searchPattern': search_pattern,
@@ -76,15 +77,11 @@ def get_documents(connection, search_term, certified_status, search_pattern='EXA
             'fields': fields
         },
     )
-    if not response.ok:
-        if error_msg is None:
-            error_msg = "Error getting available documents"
-        response_handler(response, error_msg)
-    return response
 
 
-def get_documents_async(future_session, connection, search_term, certified_status,
-                        search_pattern='EXACTLY', offset=0, limit=-1, fields=None, error_msg=None):
+def get_documents_async(future_session: "FuturesSession", connection, search_term,
+                        certified_status, search_pattern='EXACTLY', offset=0, limit=-1,
+                        fields=None, error_msg=None):
     """Get the list of available documents asynchronously.
 
     Args:
@@ -94,7 +91,7 @@ def get_documents_async(future_session, connection, search_term, certified_statu
     Returns:
         Complete HTTP response object.
     """
-    url = connection.base_url + '/api/documents/'
+    url = f'{connection.base_url}/api/documents/'
     params = {
         'searchTerm': search_term,
         'searchPattern': search_pattern,
@@ -106,6 +103,7 @@ def get_documents_async(future_session, connection, search_term, certified_statu
     return future
 
 
+@ErrorHandler(err_msg='Error getting document {document_id}')
 def get_document_status(connection, document_id, instance_id, error_msg=None):
     """Get the status of a document or dossier instance.
 
@@ -119,14 +117,10 @@ def get_document_status(connection, document_id, instance_id, error_msg=None):
         Complete HTTP response object.
     """
     url = f"{connection.base_url}​/api​/documents​/{document_id}​/instances​/{instance_id}​/status"
-    response = connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error getting document {document_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
 
 
+@ErrorHandler(err_msg='Error getting prompts for document {document_id}')
 def get_prompts_for_instance(connection, document_id, instance_id, error_msg=None):
     """Get the collection of prompts and their respective definitions from a
     document/dossier instance.
@@ -141,14 +135,10 @@ def get_prompts_for_instance(connection, document_id, instance_id, error_msg=Non
         Complete HTTP response object.
     """
     url = f"{connection.base_url}​/api​/documents​/{document_id}​/instances​/{instance_id}​/prompts"  # noqa
-    response = connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error getting prompts for document {document_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
 
 
+@ErrorHandler(err_msg='Error getting attribute element for prompt {prompt_identifier}')
 def get_attribute_element_for_prompt(connection, document_id, instance_id, prompt_identifier,
                                      error_msg=None):
     """Get available attribute element for document/dossier's attribute element
@@ -165,14 +155,10 @@ def get_attribute_element_for_prompt(connection, document_id, instance_id, promp
         Complete HTTP response object.
     """
     url = f"{connection.base_url}​/api​/documents​/{document_id}​/instances​/{instance_id}​/prompts​/{prompt_identifier}​/elements"  # noqa
-    response = connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error getting attribute element for prompt {prompt_identifier}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
 
 
+@ErrorHandler(err_msg='Error getting available object for prompt {prompt_identifier}')
 def get_available_object(connection, document_id, instance_id, prompt_identifier, error_msg=None):
     """Get available object for answering all kinds of prompts.
 
@@ -187,14 +173,10 @@ def get_available_object(connection, document_id, instance_id, prompt_identifier
         Complete HTTP response object.
     """
     url = f"{connection.base_url}/api​/documents​/{document_id}​/instances​/{instance_id}​/prompts​/{prompt_identifier}​/objects"  # noqa
-    response = connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error getting available object for prompt {prompt_identifier}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
 
 
+@ErrorHandler(err_msg='Error exporting visualization for document {document_id} to PDF file.')
 def export_visualization_to_pdf(connection, document_id, instance_id, node_key, body,
                                 error_msg=None):
     """Export a single visualization from a specific document instance to a PDF
@@ -211,14 +193,10 @@ def export_visualization_to_pdf(connection, document_id, instance_id, node_key,
         Complete HTTP response object.
     """
     url = f"{connection.base_url}​/api​/documents​/{document_id}​/instances​/{instance_id}​/visualizations​/{node_key}​/pdf"  # noqa
-    response = connection.session.post(url=url, headers={'X-MSTR-ProjectID': None}, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error exporting visualization for document {document_id} to PDF file"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.post(url=url, headers={'X-MSTR-ProjectID': None}, json=body)
 
 
+@ErrorHandler(err_msg='Error exporting visualization for document {document_id} to CSV file.')
 def export_visualization_to_csv(connection, document_id, instance_id, node_key, body,
                                 error_msg=None):
     """Export a single visualization from a specific document instance to a CSV
@@ -236,14 +214,10 @@ def export_visualization_to_csv(connection, document_id, instance_id, node_key,
         Complete HTTP response object.
     """
     url = f"{connection.base_url}​/api​/documents​/{document_id}​/instances​/{instance_id}​/visualizations​/{node_key}​/csv"  # noqa
-    response = connection.session.post(url=url, headers={'X-MSTR-ProjectID': None}, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error exporting visualization for document {document_id} to CSV file"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.post(url=url, headers={'X-MSTR-ProjectID': None}, json=body)
 
 
+@ErrorHandler(err_msg='Error exporting document {document_id} to PDF file.')
 def export_document_to_pdf(connection, document_id, instance_id, body, error_msg=None):
     """Export a specific document instance to a PDF file.
 
@@ -258,14 +232,10 @@ def export_document_to_pdf(connection, document_id, instance_id, body, error_msg
         Complete HTTP response object.
     """
     url = f"{connection.base_url}​/api​/documents​/{document_id}​/instances​/{instance_id}​/pdf"
-    response = connection.session.post(url=url, headers={'X-MSTR-ProjectID': None}, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error exporting document {document_id} to PDF file"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.post(url=url, headers={'X-MSTR-ProjectID': None}, json=body)
 
 
+@ErrorHandler(err_msg='Error exporting document {document_id} to .mstr file')
 def export_document_to_mstr(connection, document_id, instance_id, body, error_msg=None):
     """Export a specific document in a specific project to an .mstr file.
 
@@ -280,14 +250,10 @@ def export_document_to_mstr(connection, document_id, instance_id, body, error_ms
         Complete HTTP response object.
     """
     url = f"{connection.base_url}​/api​/documents​/{document_id}​/instances​/{instance_id}​/mstr"
-    response = connection.session.post(url=url, headers={'X-MSTR-ProjectID': None}, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error exporting document {document_id} to .mstr file"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.post(url=url, headers={'X-MSTR-ProjectID': None}, json=body)
 
 
+@ErrorHandler(err_msg='Error exporting document {document_id} to Excel file.')
 def export_document_to_excel(connection, document_id, instance_id, body, error_msg=None):
     """Export a document from a specific document instance to an Excel file.
 
@@ -302,14 +268,10 @@ def export_document_to_excel(connection, document_id, instance_id, body, error_m
         Complete HTTP response object.
     """
     url = f"{connection.base_url}​/api​/documents​/{document_id}​/instances​/{instance_id}​/excel"
-    response = connection.session.post(url=url, headers={'X-MSTR-ProjectID': None}, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error exporting document {document_id} to Excel file"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.post(url=url, headers={'X-MSTR-ProjectID': None}, json=body)
 
 
+@ErrorHandler(err_msg='Error setting document {document_id} to prompt status.')
 def set_document_to_prompt_status(connection, document_id, instance_id, error_msg=None):
     """Export a document from a specific document instance to an Excel file.
 
@@ -323,14 +285,10 @@ def set_document_to_prompt_status(connection, document_id, instance_id, error_ms
         Complete HTTP response object.
     """
     url = f"{connection.base_url}​/api​/documents​/{document_id}​/instances​/{instance_id}​/rePrompt"  # noqa
-    response = connection.session.post(url=url, headers={'X-MSTR-ProjectID': None})
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error setting document {document_id} to prompt status"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.post(url=url, headers={'X-MSTR-ProjectID': None})
 
 
+@ErrorHandler(err_msg='Error getting cubes used by document {document_id}')
 def get_cubes_used_by_document(connection, document_id, error_msg=None):
     """Get the cubes used by a document in a specific project, either directly
     or indirectly.
@@ -344,14 +302,10 @@ def get_cubes_used_by_document(connection, document_id, error_msg=None):
         Complete HTTP response object.
     """
     url = f"{connection.base_url}/api/documents/{document_id}/cubes"
-    response = connection.session.get(url=url)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error getting cubes used by document {document_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url)
 
 
+@ErrorHandler(err_msg='Error overwriting document {document_id}')
 def overwrite_document(connection, document_id, instance_id, error_msg=None):
     """Save a document instance by overwriting an existing document.
 
@@ -365,14 +319,10 @@ def overwrite_document(connection, document_id, instance_id, error_msg=None):
         Complete HTTP response object.
     """
     url = f"{connection.base_url}​/api​/documents​/{document_id}​/instances​/{instance_id}​/save"
-    response = connection.session.post(url=url, headers={'X-MSTR-ProjectID': None})
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error overwriting document {document_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.post(url=url, headers={'X-MSTR-ProjectID': None})
 
 
+@ErrorHandler(err_msg='Error saving document {document_id}')
 def save_document_as(connection, document_id, instance_id, error_msg=None):
     """Save a document instance by creating a new document.
 
@@ -386,14 +336,10 @@ def save_document_as(connection, document_id, instance_id, error_msg=None):
         Complete HTTP response object.
     """
     url = f"{connection.base_url}/api​/documents​/{document_id}​/instances​/{instance_id}​/saveAs"  # noqa
-    response = connection.session.post(url=url, headers={'X-MSTR-ProjectID': None})
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error saving document {document_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.post(url=url, headers={'X-MSTR-ProjectID': None})
 
 
+@ErrorHandler(err_msg='Error creating instance for document {document_id}')
 def create_new_document_instance(connection, document_id, body, error_msg=None):
     """Execute a specific document in a specific project and create an instance
     of the document.
@@ -407,14 +353,10 @@ def create_new_document_instance(connection, document_id, body, error_msg=None):
         Complete HTTP response object.
     """
     url = f"{connection.base_url}/api/documents/{document_id}/instances"
-    response = connection.session.post(url=url, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error creating instance for document {document_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.post(url=url, json=body)
 
 
+@ErrorHandler(err_msg='Error deleting document instance {instance_id}')
 def delete_document_instance(connection, document_id, instance_id, error_msg=None):
     """Save a document instance by overwriting an existing document.
 
@@ -428,14 +370,10 @@ def delete_document_instance(connection, document_id, instance_id, error_msg=Non
         Complete HTTP response object.
     """
     url = f"{connection.base_url}​/api​/documents​/{document_id}​/instances​/{instance_id}​"
-    response = connection.session.delete(url=url, headers={'X-MSTR-ProjectID': None})
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error deleting document instance {instance_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.delete(url=url, headers={'X-MSTR-ProjectID': None})
 
 
+@ErrorHandler(err_msg='Error refreshing document instance {instance_id}')
 def refresh_document_instance(connection, document_id, instance_id, error_msg=None):
     """Execute a specific document in a specific project and create an instance
     of the document.
@@ -450,14 +388,10 @@ def refresh_document_instance(connection, document_id, instance_id, error_msg=No
         Complete HTTP response object.
     """
     url = f"{connection.base_url}​/api​/documents​/{document_id}​/instances/{instance_id}/refresh​"
-    response = connection.session.put(url=url, headers={'X-MSTR-ProjectID': None})
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error refreshing document instance {instance_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.put(url=url, headers={'X-MSTR-ProjectID': None})
 
 
+@ErrorHandler(err_msg='Error getting collection of prompts for document {document_id}')
 def get_prompts(connection, document_id, error_msg=None):
     """Get the collection of prompts and their respective definitions from a
     document/dossier definition.
@@ -471,14 +405,10 @@ def get_prompts(connection, document_id, error_msg=None):
         Complete HTTP response object.
     """
     url = f"{connection.base_url}​/api​/documents​/{document_id}​/prompts"
-    response = connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error getting collection of prompts for document {document_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
 
 
+@ErrorHandler(err_msg='Error answering prompt for instance {instance_id}')
 def answer_prompts(connection, document_id, instance_id, body, error_msg=None):
     """Answer specified prompts on the document/dossier instance, prompts can
     either be answered with default answers(if available), the appropriate
@@ -495,14 +425,10 @@ def answer_prompts(connection, document_id, instance_id, body, error_msg=None):
         Complete HTTP response object.
     """
     url = f"{connection.base_url}/api​/documents​/{document_id}​/instances/{instance_id}/prompts/answers"  # noqa
-    response = connection.session.put(url=url, headers={'X-MSTR-ProjectID': None}, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error answering prompt for instance {instance_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.put(url=url, headers={'X-MSTR-ProjectID': None}, json=body)
 
 
+@ErrorHandler(err_msg='Error retrieving document shortcut for document {document_id}')
 def get_document_shortcut(connection, document_id, instance_id, error_msg=None):
     """Retrieve a published shortcut from a specific document instance.
 
@@ -516,14 +442,10 @@ def get_document_shortcut(connection, document_id, instance_id, error_msg=None):
         Complete HTTP response object.
     """
     url = f"{connection.base_url}/api​/documents​/{document_id}​/instances/{instance_id}/shortcut"
-    response = connection.session.get(url=url)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error retrieving document shortcut for document {document_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url)
 
 
+@ErrorHandler(err_msg='Error creating instance for dossier {dossier_id}')
 def create_dossier_instance(connection, dossier_id, body, error_msg=None):
     """Execute a specific dossier and create an instance of the dossier.
 
@@ -537,14 +459,10 @@ def create_dossier_instance(connection, dossier_id, body, error_msg=None):
         Complete HTTP response object.
     """
     url = f"{connection.base_url}​/api​/dossiers/{dossier_id}​/instances​"
-    response = connection.session.post(url=url, headers={'X-MSTR-ProjectID': None}, json=body)
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error creating instance for dossier {dossier_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.post(url=url, headers={'X-MSTR-ProjectID': None}, json=body)
 
 
+@ErrorHandler(err_msg='Error getting hierarchy for dossier {id}')
 def get_dossier_hierarchy(connection, id, error_msg=None):
     """Get the hierarchy of a specific dossier in a specific project.
 
@@ -557,14 +475,10 @@ def get_dossier_hierarchy(connection, id, error_msg=None):
         Complete HTTP response object.
     """
     url = f"{connection.base_url}/api/v2/dossiers/{id}/definition/"
-    response = connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error getting hierarchy for dossier {id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
 
 
+@ErrorHandler(err_msg='Error getting definition of document {id}')
 def get_document_definition(connection, id, error_msg=None):
     """Get the hierarchy of a specific dossier in a specific project.
 
@@ -577,14 +491,10 @@ def get_document_definition(connection, id, error_msg=None):
         Complete HTTP response object.
     """
     url = f"{connection.base_url}/api/v2/documents/{id}"
-    response = connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error getting definition of document {id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
 
 
+@ErrorHandler(err_msg='Error getting dossier hierarchy from instance {instance_id}')
 def get_dossier_hierarchy_from_instance(connection, dossier_id, instance_id, error_msg=None):
     """Get the hierarchy of a specific dossier in a specific project from
     instance.
@@ -599,14 +509,10 @@ def get_dossier_hierarchy_from_instance(connection, dossier_id, instance_id, err
         Complete HTTP response object.
     """
     url = f"{connection.base_url}/api​/v2/dossiers/{dossier_id}/​instances/{instance_id}/definition"  # noqa
-    response = connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error getting dossier hierarchy from instance {instance_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
 
 
+@ErrorHandler(err_msg='Error getting definition and results for dossier {dossier_id}')
 def get_definition_and_results_of_visualization(connection, dossier_id, instance_id, chapter_key,
                                                 visualization_key, error_msg=None):
     """Get the definition and data result of a grid/graph visualization in a
@@ -622,9 +528,4 @@ def get_definition_and_results_of_visualization(connection, dossier_id, instance
         Complete HTTP response object.
     """
     url = f"{connection.base_url}/api/​v2/dossiers​/{dossier_id}​/instances​/{instance_id}​/chapters​/{chapter_key}​/visualizations​/{visualization_key}"  # noqa
-    response = connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
-    if not response.ok:
-        if error_msg is None:
-            error_msg = f"Error getting definition and results for dossier {dossier_id}"
-        response_handler(response, error_msg)
-    return response
+    return connection.session.get(url=url, headers={'X-MSTR-ProjectID': None})
