@@ -1,10 +1,16 @@
+from typing import TYPE_CHECKING
+
 from packaging import version
 
-from mstrio.utils.helper import response_handler
+from mstrio.utils.error_handlers import ErrorHandler
+
+if TYPE_CHECKING:
+    from requests_futures.sessions import FuturesSession
 
 CUBE_FIELDS = '-data.metricValues.extras,-data.metricValues.formatted'
 
 
+@ErrorHandler(err_msg='Error getting report {report_id} definition. Check report ID.')
 def report_definition(connection, report_id):
     """Get the definition of a specific report, including attributes and
     metrics. This in-memory report definition provides information about all
@@ -19,14 +25,12 @@ def report_definition(connection, report_id):
     Returns:
         Complete HTTP response object.
     """
-    connection._validate_application_selected()
-    response = connection.session.get(url=connection.base_url + '/api/v2/reports/' + report_id)
-    if not response.ok:
-        response_handler(response, "Error getting report definition. Check report ID.")
-    return response
+    connection._validate_project_selected()
+    return connection.session.get(url=f'{connection.base_url}/api/v2/reports/{report_id}')
 
 
-def report_instance(connection, report_id, body={}, offset=0, limit=5000):
+@ErrorHandler(err_msg='Error getting report {report_id} contents.')
+def report_instance(connection, report_id, body=None, offset=0, limit=5000):
     """Get the results of a newly created report instance. This in-memory
     report instance can be used by other requests.
 
@@ -46,20 +50,21 @@ def report_instance(connection, report_id, body={}, offset=0, limit=5000):
     Returns:
         Complete HTTP response object.
     """
+    connection._validate_project_selected()
+    if body is None:
+        body = {}
     params = {'offset': offset, 'limit': limit}
     if version.parse(connection.iserver_version) >= version.parse("11.2.0200"):
         params['fields'] = CUBE_FIELDS
 
-    response = connection.session.post(
-        url=connection.base_url + '/api/v2/reports/' + report_id + '/instances/',
+    return connection.session.post(
+        url=f'{connection.base_url}/api/v2/reports/{report_id}/instances/',
         json=body,
         params=params,
     )
-    if not response.ok:
-        response_handler(response, "Error getting report contents.")
-    return response
 
 
+@ErrorHandler(err_msg='Error getting cube contents.')
 def report_instance_id(connection, report_id, instance_id, offset=0, limit=5000):
     """Get the results of a previously created report instance, using the in-
     memory report instance created by a POST /api/reports/{reportId}/instances
@@ -83,21 +88,19 @@ def report_instance_id(connection, report_id, instance_id, offset=0, limit=5000)
     Returns:
         Complete HTTP response object.
     """
+    connection._validate_project_selected()
     params = {'offset': offset, 'limit': limit}
     if version.parse(connection.iserver_version) >= version.parse("11.2.0200"):
         params['fields'] = CUBE_FIELDS
 
-    response = connection.session.get(
-        url=connection.base_url + '/api/v2/reports/' + report_id + '/instances/' + instance_id,
+    return connection.session.get(
+        url=f'{connection.base_url}/api/v2/reports/{report_id}/instances/{instance_id}',
         params=params,
     )
-    if not response.ok:
-        response_handler(response, "Error getting cube contents.")
-    return response
 
 
-def report_instance_id_coroutine(future_session, connection, report_id, instance_id, offset=0,
-                                 limit=5000):
+def report_instance_id_coroutine(future_session: "FuturesSession", connection, report_id,
+                                 instance_id, offset=0, limit=5000):
     """Get the future of a previously created instance for a specific report
     asynchroneously, using the in-memory instance created by report_instance().
 
@@ -108,11 +111,12 @@ def report_instance_id_coroutine(future_session, connection, report_id, instance
     if version.parse(connection.iserver_version) >= version.parse("11.2.0200"):
         params['fields'] = CUBE_FIELDS
 
-    url = connection.base_url + '/api/v2/reports/' + report_id + '/instances/' + instance_id
+    url = f'{connection.base_url}/api/v2/reports/{report_id}/instances/{instance_id}'
     future = future_session.get(url, params=params)
     return future
 
 
+@ErrorHandler(err_msg='Error retrieving attribute {attribute_id} elements.')
 def report_single_attribute_elements(connection, report_id, attribute_id, offset=0, limit=200000):
     """Get elements of a specific attribute of a specific report.
 
@@ -132,25 +136,23 @@ def report_single_attribute_elements(connection, report_id, attribute_id, offset
     Returns:
         Complete HTTP response object
     """
-
-    response = connection.session.get(
-        url=connection.base_url + '/api/reports/' + report_id + '/attributes/' + attribute_id
-        + '/elements',
+    url = f'{connection.base_url}/api/reports/{report_id}/attributes/{attribute_id}/elements'
+    return connection.session.get(
+        url=url,
         params={
             'offset': offset,
             'limit': limit
         },
     )
-    if not response.ok:
-        response_handler(response, "Error retrieving attribute " + attribute_id + " elements")
-    return response
 
 
-def report_single_attribute_elements_coroutine(future_session, connection, report_id, attribute_id,
-                                               offset=0, limit=200000):
+def report_single_attribute_elements_coroutine(future_session: "FuturesSession", connection,
+                                               report_id, attribute_id, offset=0, limit=200000):
     """Get elements of a specific attribute of a specific report.
 
     Args:
+        future_session(object): Future Session object to call MicroStrategy REST
+            Server asynchronously
         connection: MicroStrategy REST API connection object.
         report_id (str): Unique ID of the report you wish to extract information
             from.
@@ -166,7 +168,61 @@ def report_single_attribute_elements_coroutine(future_session, connection, repor
     Returns:
         Complete Future object
     """
-    url = (connection.base_url + '/api/reports/' + report_id + '/attributes/' + attribute_id
-           + '/elements')
+    url = f'{connection.base_url}/api/reports/{report_id}/attributes/{attribute_id}/elements'
     future = future_session.get(url, params={'offset': offset, 'limit': limit})
     return future
+
+
+@ErrorHandler(err_msg='Error getting collection of prompts for report {report_id}')
+def get_report_prompts(connection, report_id, closed=None, fields=None):
+    """Get the collection of prompts and their respective definitions from a
+    report.
+
+    Args:
+        connection: MicroStrategy REST API connection object.
+        report_id (str): Unique ID of the report you wish
+            to extract information from.
+        closed: Prompt status, true means get closed prompt,
+            false means get open prompt
+        fields: Comma-separated, top-level field whitelist
+            that allows the client to selectively retrieve
+            part of the response model.
+
+    """
+    url = f'{connection.base_url}/api/reports/{report_id}/prompts'
+    return connection.session.get(
+        url=url,
+        params={
+            'closed': closed,
+            'fields': fields
+        },
+    )
+
+
+@ErrorHandler(err_msg='Error getting prompted report {report_id} instance.')
+def get_prompted_instance(connection, report_id, instance_id, closed=None, fields=None):
+    """Get the collection of prompts and their respective definitions from a
+    report instance. This endpoint will return data only when the report
+    instance has prompt which need to be answered.
+
+    Args:
+        connection: MicroStrategy REST API connection object.
+        report_id (str): Unique ID of the report you wish
+            to extract information from.
+        instance_id (str): Unique ID of the in-memory instance of a published
+            report.
+        closed(bool): Prompt status, true means get closed prompt,
+            false means get open prompt
+        fields: Comma-separated, top-level field whitelist
+            that allows the client to selectively retrieve
+            part of the response model.
+
+    """
+    url = f'{connection.base_url}/api/reports/{report_id}/instances/{instance_id}/prompts'
+    return connection.session.get(
+        url=url,
+        params={
+            'closed': closed,
+            'fields': fields
+        },
+    )
